@@ -3,6 +3,7 @@
  * Copyright (c) 2015-2022 LunarG, Inc.
  * Copyright (C) 2015-2022 Google Inc.
  * Modifications Copyright (C) 2020 Advanced Micro Devices, Inc. All rights reserved.
+ * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,10 +24,12 @@
  * Author: Dave Houlton <daveh@lunarg.com>
  * Author: John Zulauf <jzulauf@lunarg.com>
  * Author: Tobias Hector <tobias.hector@amd.com>
+ * Author: Daniel Rakos <daniel.rakos@rastergrid.com>
  */
 #pragma once
 #include "base_node.h"
 #include "query_state.h"
+#include "video_session_state.h"
 #include "command_validation.h"
 #include "hash_vk_types.h"
 #include "subresource_adapter.h"
@@ -35,10 +38,13 @@
 #include "device_state.h"
 #include "descriptor_sets.h"
 #include "qfo_transfer.h"
+#include "vk_layer_data.h"
 
 struct SUBPASS_INFO;
 class FRAMEBUFFER_STATE;
 class RENDER_PASS_STATE;
+class VIDEO_SESSION_STATE;
+class VIDEO_SESSION_PARAMETERS_STATE;
 class CoreChecks;
 class ValidationStateTracker;
 
@@ -165,7 +171,6 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     // Track if certain commands have been called at least once in lifetime of the command buffer
     // primary command buffers values are set true if a secondary command buffer has a command
     bool has_draw_cmd;
-    bool has_draw_cmd_in_current_render_pass;
     bool has_dispatch_cmd;
     bool has_trace_rays_cmd;
     bool has_build_as_cmd;
@@ -176,10 +181,34 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     bool pipeline_bound = false;  // True if CmdBindPipeline has been called on this command buffer, false otherwise
     typedef uint64_t ImageLayoutUpdateCount;
     ImageLayoutUpdateCount image_layout_change_count;  // The sequence number for changes to image layout (for cached validation)
+
+    // Dynamic State
     CBDynamicFlags status;                             // Track status of various bindings on cmd buffer
     CBDynamicFlags static_status;                      // All state bits provided by current graphics pipeline
                                                        // rather than dynamic state
     CBDynamicFlags dynamic_status;                     // dynamic state set up in pipeline
+    struct DynamicStateValue {
+        // VK_DYNAMIC_STATE_STENCIL_WRITE_MASK
+        uint32_t write_mask_front;
+        uint32_t write_mask_back;
+        // VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE
+        bool depth_write_enable;
+        // VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE
+        bool depth_test_enable;
+        // VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE
+        bool stencil_test_enable;
+        // VK_DYNAMIC_STATE_STENCIL_OP
+        VkStencilOp fail_op_front;
+        VkStencilOp pass_op_front;
+        VkStencilOp depth_fail_op_front;
+        VkStencilOp fail_op_back;
+        VkStencilOp pass_op_back;
+        VkStencilOp depth_fail_op_back;
+        // VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT
+        // maxDiscardRectangles is at max 8 on all known implementations currently
+        std::bitset<32> discard_rectangles;
+    } dynamic_state_value;
+
     std::string begin_rendering_func_name;
     // Currently storing "lastBound" objects on per-CB basis
     //  long-term may want to create caches of "lastBound" states and could have
@@ -191,9 +220,10 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     std::shared_ptr<const CMD_BUFFER_STATE> shared_from_this() const { return SharedFromThisImpl(this); }
     std::shared_ptr<CMD_BUFFER_STATE> shared_from_this() { return SharedFromThisImpl(this); }
 
+    using DescriptorBindingInfo = std::pair<const uint32_t, DescriptorRequirement>;
     struct CmdDrawDispatchInfo {
         CMD_TYPE cmd_type;
-        std::vector<std::pair<const uint32_t, DescriptorRequirement>> binding_infos;
+        std::vector<DescriptorBindingInfo> binding_infos;
         VkFramebuffer framebuffer;
         std::shared_ptr<std::vector<SUBPASS_INFO>> subpasses;
         std::shared_ptr<std::vector<IMAGE_VIEW_STATE *>> attachments;
@@ -250,7 +280,6 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     uint32_t active_render_pass_device_mask;
     uint32_t activeSubpass;
     std::shared_ptr<FRAMEBUFFER_STATE> activeFramebuffer;
-    layer_data::unordered_set<std::shared_ptr<FRAMEBUFFER_STATE>> framebuffers;
     // Unified data structs to track objects bound to this command buffer as well as object
     //  dependencies that have been broken : either destroyed objects, or updated descriptor sets
     layer_data::unordered_set<std::shared_ptr<BASE_NODE>> object_bindings;
@@ -284,9 +313,10 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     // Validation functions run when secondary CB is executed in primary
     std::vector<std::function<bool(const CMD_BUFFER_STATE &secondary, const CMD_BUFFER_STATE *primary, const FRAMEBUFFER_STATE *)>>
         cmd_execute_commands_functions;
-    std::vector<std::function<bool(CMD_BUFFER_STATE &cb, bool do_validate, EventToStageMap *localEventToStageMap)>> eventUpdates;
-    std::vector<std::function<bool(CMD_BUFFER_STATE &cb, bool do_validate, VkQueryPool &firstPerfQueryPool, uint32_t perfQueryPass,
-                                   QueryMap *localQueryToStateMap)>>
+    std::vector<std::function<bool(CMD_BUFFER_STATE &cb_state, bool do_validate, EventToStageMap *localEventToStageMap)>>
+        eventUpdates;
+    std::vector<std::function<bool(CMD_BUFFER_STATE &cb_state, bool do_validate, VkQueryPool &firstPerfQueryPool,
+                                   uint32_t perfQueryPass, QueryMap *localQueryToStateMap)>>
         queryUpdates;
     layer_data::unordered_map<const cvdescriptorset::DescriptorSet *, cvdescriptorset::DescriptorSet::CachedValidation>
         descriptorset_cache;
@@ -307,12 +337,20 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     // Used for Best Practices tracking
     uint32_t small_indexed_draw_call_count;
 
+    // Video coding related state tracking
+    std::shared_ptr<VIDEO_SESSION_STATE> bound_video_session;
+    std::shared_ptr<VIDEO_SESSION_PARAMETERS_STATE> bound_video_session_parameters;
+    BoundVideoPictureResources bound_video_picture_resources;
+    VideoSessionUpdateMap video_session_updates;
+
     bool transform_feedback_active{false};
     bool conditional_rendering_active{false};
     bool conditional_rendering_inside_render_pass{false};
     uint32_t conditional_rendering_subpass{0};
     uint32_t dynamicColorWriteEnableAttachmentCount{0};
-    mutable ReadWriteLock lock;
+    std::vector<VkDescriptorBufferBindingInfoEXT> descriptor_buffer_binding_info;
+
+    mutable std::shared_mutex lock;
     ReadLockGuard ReadLock() const { return ReadLockGuard(lock); }
     WriteLockGuard WriteLock() { return WriteLockGuard(lock); }
 
@@ -334,6 +372,12 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         auto base = std::static_pointer_cast<BASE_NODE>(child_node);
         AddChild(base);
     }
+    template <typename StateObject>
+    void AddChildren(layer_data::span<std::shared_ptr<StateObject>> &child_nodes) {
+        for (auto &child_node : child_nodes) {
+            AddChild(child_node);
+        }
+    }
 
     void RemoveChild(std::shared_ptr<BASE_NODE> &base_node);
     template <typename StateObject>
@@ -376,12 +420,12 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     void GetCurrentPipelineAndDesriptorSets(VkPipelineBindPoint pipelineBindPoint, const PIPELINE_STATE **rtn_pipe,
                                             const std::vector<LAST_BOUND_STATE::PER_SET> **rtn_sets) const {
         const auto lv_bind_point = ConvertToLvlBindPoint(pipelineBindPoint);
-        const auto &last_bound_it = lastBound[lv_bind_point];
-        if (!last_bound_it.IsUsing()) {
+        const auto &last_bound = lastBound[lv_bind_point];
+        if (!last_bound.IsUsing()) {
             return;
         }
-        *rtn_pipe = last_bound_it.pipeline_state;
-        *rtn_sets = &(last_bound_it.per_set);
+        *rtn_pipe = last_bound.pipeline_state;
+        *rtn_sets = &(last_bound.per_set);
     }
 
     VkQueueFlags GetQueueFlags() const {
@@ -412,14 +456,23 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
 
     void BeginRendering(CMD_TYPE cmd_type, const VkRenderingInfo *pRenderingInfo);
 
-    void ExecuteCommands(uint32_t commandBuffersCount, const VkCommandBuffer *pCommandBuffers);
+    void BeginVideoCoding(const VkVideoBeginCodingInfoKHR *pBeginInfo);
+    void EndVideoCoding(const VkVideoEndCodingInfoKHR *pEndCodingInfo);
+    void ControlVideoCoding(const VkVideoCodingControlInfoKHR *pControlInfo);
+    void DecodeVideo(const VkVideoDecodeInfoKHR *pDecodeInfo);
 
-    void UpdateLastBoundDescriptorSets(VkPipelineBindPoint pipeline_bind_point, const PIPELINE_LAYOUT_STATE *pipeline_layout,
+    void ExecuteCommands(layer_data::span<const VkCommandBuffer> secondary_command_buffers);
+
+    void UpdateLastBoundDescriptorSets(VkPipelineBindPoint pipeline_bind_point, const PIPELINE_LAYOUT_STATE &pipeline_layout,
                                        uint32_t first_set, uint32_t set_count, const VkDescriptorSet *pDescriptorSets,
                                        std::shared_ptr<cvdescriptorset::DescriptorSet> &push_descriptor_set,
                                        uint32_t dynamic_offset_count, const uint32_t *p_dynamic_offsets);
 
-    void PushDescriptorSetState(VkPipelineBindPoint pipelineBindPoint, PIPELINE_LAYOUT_STATE *pipeline_layout, uint32_t set,
+    void UpdateLastBoundDescriptorBuffers(VkPipelineBindPoint pipeline_bind_point, const PIPELINE_LAYOUT_STATE &pipeline_layout,
+                                          uint32_t first_set, uint32_t set_count, const uint32_t *buffer_indicies,
+                                          const VkDeviceSize *buffer_offsets);
+
+    void PushDescriptorSetState(VkPipelineBindPoint pipelineBindPoint, const PIPELINE_LAYOUT_STATE &pipeline_layout, uint32_t set,
                                 uint32_t descriptorWriteCount, const VkWriteDescriptorSet *pDescriptorWrites);
 
     void UpdateDrawCmd(CMD_TYPE cmd_type);
@@ -428,9 +481,9 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
     void UpdatePipelineState(CMD_TYPE cmd_type, const VkPipelineBindPoint bind_point);
 
     virtual void RecordCmd(CMD_TYPE cmd_type);
-    void RecordStateCmd(CMD_TYPE cmd_type, CB_DYNAMIC_STATUS state);
+    void RecordStateCmd(CMD_TYPE cmd_type, CBDynamicStatus state);
     void RecordStateCmd(CMD_TYPE cmd_type, CBDynamicFlags const &state_bits);
-    void RecordColorWriteEnableStateCmd(CMD_TYPE cmd_type, CB_DYNAMIC_STATUS state, uint32_t attachment_count);
+    void RecordColorWriteEnableStateCmd(CMD_TYPE cmd_type, CBDynamicStatus state, uint32_t attachment_count);
     void RecordTransferCmd(CMD_TYPE cmd_type, std::shared_ptr<BINDABLE> &&buf1, std::shared_ptr<BINDABLE> &&buf2 = nullptr);
     void RecordSetEvent(CMD_TYPE cmd_type, VkEvent event, VkPipelineStageFlags2KHR stageMask);
     void RecordResetEvent(CMD_TYPE cmd_type, VkEvent event, VkPipelineStageFlags2KHR stageMask);
@@ -504,6 +557,9 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
         pipeline_bound = true;
     }
 
+  private:
+    void ResetCBState();
+
   protected:
     void NotifyInvalidate(const BASE_NODE::NodeList &invalid_nodes, bool unlink) override;
     void UpdateAttachmentsView(const VkRenderPassBeginInfo *pRenderPassBegin);
