@@ -56,7 +56,8 @@ void DESCRIPTOR_POOL_STATE::Allocate(const VkDescriptorSetAllocateInfo *alloc_in
     }
 
     const auto *variable_count_info = LvlFindInChain<VkDescriptorSetVariableDescriptorCountAllocateInfo>(alloc_info->pNext);
-    bool variable_count_valid = variable_count_info && variable_count_info->descriptorSetCount == alloc_info->descriptorSetCount;
+    const bool variable_count_valid =
+        variable_count_info && variable_count_info->descriptorSetCount == alloc_info->descriptorSetCount;
 
     // Create tracking object for each descriptor set; insert into global map and the pool's set.
     for (uint32_t i = 0; i < alloc_info->descriptorSetCount; i++) {
@@ -380,10 +381,21 @@ bool cvdescriptorset::DescriptorSetLayoutDef::IsNextBindingConsistent(const uint
     return false;
 }
 
+void cvdescriptorset::DescriptorSetLayout::SetLayoutSizeInBytes(const VkDeviceSize *layout_size_in_bytes_) {
+    if (layout_size_in_bytes_) {
+        layout_size_in_bytes = std::make_unique<VkDeviceSize>(*layout_size_in_bytes_);
+    } else {
+        layout_size_in_bytes.reset();
+    }
+}
+
+const VkDeviceSize* cvdescriptorset::DescriptorSetLayout::GetLayoutSizeInBytes() const {
+    return layout_size_in_bytes.get();
+}
+
 // If our layout is compatible with rh_ds_layout, return true.
 bool cvdescriptorset::DescriptorSetLayout::IsCompatible(DescriptorSetLayout const *rh_ds_layout) const {
-    bool compatible = (this == rh_ds_layout) || (GetLayoutDef() == rh_ds_layout->GetLayoutDef());
-    return compatible;
+    return (this == rh_ds_layout) || (GetLayoutDef() == rh_ds_layout->GetLayoutDef());
 }
 
 // The DescriptorSetLayout stores the per handle data for a descriptor set layout, and references the common defintion for the
@@ -539,7 +551,7 @@ void cvdescriptorset::DescriptorSet::PerformWriteUpdate(ValidationStateTracker *
     }
     if (update->descriptorCount) {
         some_update_ = true;
-        change_count_++;
+        ++change_count_;
     }
 
     if (!IsPushDescriptor() && !(orig_binding.binding_flags & (VK_DESCRIPTOR_BINDING_UPDATE_UNUSED_WHILE_PENDING_BIT |
@@ -559,7 +571,7 @@ void cvdescriptorset::DescriptorSet::PerformCopyUpdate(ValidationStateTracker *d
         if (src_iter.updated()) {
             dst.CopyUpdate(this, state_data_, &src, src_iter.CurrentBinding().IsBindless());
             some_update_ = true;
-            change_count_++;
+            ++change_count_;
             dst_iter.updated(true);
         } else {
             dst_iter.updated(false);
@@ -573,13 +585,13 @@ void cvdescriptorset::DescriptorSet::PerformCopyUpdate(ValidationStateTracker *d
 }
 
 // Update the drawing state for the affected descriptors.
-// Set cb_node to this set and this set to cb_node.
+// Set cb_state to this set and this set to cb_state.
 // Add the bindings of the descriptor
 // Set the layout based on the current descriptor layout (will mask subsequent layer mismatch errors)
 // TODO: Modify the UpdateDrawState virtural functions to *only* set initial layout and not change layouts
-// Prereq: This should be called for a set that has been confirmed to be active for the given cb_node, meaning it's going
-//   to be used in a draw by the given cb_node
-void cvdescriptorset::DescriptorSet::UpdateDrawState(ValidationStateTracker *device_data, CMD_BUFFER_STATE *cb_node,
+// Prereq: This should be called for a set that has been confirmed to be active for the given cb_state, meaning it's going
+//   to be used in a draw by the given cb_state
+void cvdescriptorset::DescriptorSet::UpdateDrawState(ValidationStateTracker *device_data, CMD_BUFFER_STATE *cb_state,
                                                      CMD_TYPE cmd_type, const PIPELINE_STATE *pipe,
                                                      const BindingReqMap &binding_req_map) {
     // Descriptor UpdateDrawState only call image layout validation callbacks. If it is disabled, skip the entire loop.
@@ -605,21 +617,21 @@ void cvdescriptorset::DescriptorSet::UpdateDrawState(ValidationStateTracker *dev
             case Image: {
                 auto *image_binding = static_cast<ImageBinding *>(binding);
                 for (uint32_t i = 0; i < image_binding->count; ++i) {
-                    image_binding->descriptors[i].UpdateDrawState(device_data, cb_node);
+                    image_binding->descriptors[i].UpdateDrawState(device_data, cb_state);
                 }
                 break;
             }
             case ImageSampler: {
                 auto *image_binding = static_cast<ImageSamplerBinding *>(binding);
                 for (uint32_t i = 0; i < image_binding->count; ++i) {
-                    image_binding->descriptors[i].UpdateDrawState(device_data, cb_node);
+                    image_binding->descriptors[i].UpdateDrawState(device_data, cb_state);
                 }
                 break;
             }
             case Mutable: {
                 auto *mutable_binding = static_cast<MutableBinding *>(binding);
                 for (uint32_t i = 0; i < mutable_binding->count; ++i) {
-                    mutable_binding->descriptors[i].UpdateDrawState(device_data, cb_node);
+                    mutable_binding->descriptors[i].UpdateDrawState(device_data, cb_state);
                 }
                 break;
             }
@@ -630,12 +642,12 @@ void cvdescriptorset::DescriptorSet::UpdateDrawState(ValidationStateTracker *dev
 
     if (cmd_info.binding_infos.size() > 0) {
         cmd_info.cmd_type = cmd_type;
-        if (cb_node->activeFramebuffer) {
-            cmd_info.framebuffer = cb_node->activeFramebuffer->framebuffer();
-            cmd_info.attachments = cb_node->active_attachments;
-            cmd_info.subpasses = cb_node->active_subpasses;
+        if (cb_state->activeFramebuffer) {
+            cmd_info.framebuffer = cb_state->activeFramebuffer->framebuffer();
+            cmd_info.attachments = cb_state->active_attachments;
+            cmd_info.subpasses = cb_state->active_subpasses;
         }
-        cb_node->validate_descriptorsets_in_queuesubmit[GetSet()].emplace_back(cmd_info);
+        cb_state->validate_descriptorsets_in_queuesubmit[GetSet()].emplace_back(cmd_info);
     }
 }
 
@@ -817,11 +829,11 @@ void cvdescriptorset::ImageDescriptor::CopyUpdate(DescriptorSet *set_state, cons
     ReplaceStatePtr(set_state, image_view_state_, image_src->image_view_state_, is_bindless);
 }
 
-void cvdescriptorset::ImageDescriptor::UpdateDrawState(ValidationStateTracker *dev_data, CMD_BUFFER_STATE *cb_node) {
+void cvdescriptorset::ImageDescriptor::UpdateDrawState(ValidationStateTracker *dev_data, CMD_BUFFER_STATE *cb_state) {
     // Add binding for image
     auto iv_state = GetImageViewState();
     if (iv_state) {
-        dev_data->CallSetImageViewInitialLayoutCallback(cb_node, *iv_state, image_layout_);
+        dev_data->CallSetImageViewInitialLayoutCallback(cb_state, *iv_state, image_layout_);
     }
 }
 
@@ -897,6 +909,7 @@ void cvdescriptorset::AccelerationStructureDescriptor::CopyUpdate(DescriptorSet
         return;
     }
     auto acc_desc = static_cast<const AccelerationStructureDescriptor *>(src);
+    is_khr_ = acc_desc->is_khr_;
     if (is_khr_) {
         acc_ = acc_desc->acc_;
         ReplaceStatePtr(set_state, acc_state_, dev_data->GetConstCastShared<ACCELERATION_STRUCTURE_STATE_KHR>(acc_), is_bindless);
@@ -1074,11 +1087,11 @@ void cvdescriptorset::MutableDescriptor::CopyUpdate(DescriptorSet *set_state, co
     }
 }
 
-void cvdescriptorset::MutableDescriptor::UpdateDrawState(ValidationStateTracker *dev_data, CMD_BUFFER_STATE *cb_node) {
+void cvdescriptorset::MutableDescriptor::UpdateDrawState(ValidationStateTracker *dev_data, CMD_BUFFER_STATE *cb_state) {
     auto active_class = DescriptorTypeToClass(active_descriptor_type_);
     if (active_class == Image || active_class == ImageSampler) {
         if (image_view_state_) {
-            dev_data->CallSetImageViewInitialLayoutCallback(cb_node, *image_view_state_, image_layout_);
+            dev_data->CallSetImageViewInitialLayoutCallback(cb_state, *image_view_state_, image_layout_);
         }
     }
 }
