@@ -1,6 +1,7 @@
-/* Copyright (c) 2015-2017, 2019-2022 The Khronos Group Inc.
- * Copyright (c) 2015-2017, 2019-2022 Valve Corporation
- * Copyright (c) 2015-2017, 2019-2022 LunarG, Inc.
+/* Copyright (c) 2015-2017, 2019-2023 The Khronos Group Inc.
+ * Copyright (c) 2015-2017, 2019-2023 Valve Corporation
+ * Copyright (c) 2015-2017, 2019-2023 LunarG, Inc.
+ * Modifications Copyright (C) 2022 RasterGrid Kft.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,16 +18,18 @@
  * Author: Mark Lobodzinski <mark@lunarg.com>
  * Author: Courtney Goeltzenleuchter <courtney@LunarG.com>
  * Author: Dave Houlton <daveh@lunarg.com>
+ * Author: Daniel Rakos <daniel.rakos@rastergrid.com>
  */
 
 #pragma once
 
 #include <cassert>
 #include <cstddef>
+#include <cstring>
 #include <functional>
-#include <stdbool.h>
 #include <string>
 #include <vector>
+#include <bitset>
 #include <iomanip>
 #include "cast_utils.h"
 #include "vk_format_utils.h"
@@ -206,7 +209,7 @@ static inline uint32_t SampleCountSize(VkSampleCountFlagBits sample_count) {
 }
 
 static inline bool IsImageLayoutReadOnly(VkImageLayout layout) {
-    constexpr std::array<VkImageLayout, 7> read_only_layouts = {
+    constexpr std::array read_only_layouts = {
         VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL,
         VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL,
         VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_STENCIL_ATTACHMENT_OPTIMAL,
@@ -220,7 +223,7 @@ static inline bool IsImageLayoutReadOnly(VkImageLayout layout) {
 }
 
 static inline bool IsImageLayoutDepthReadOnly(VkImageLayout layout) {
-    constexpr std::array<VkImageLayout, 7> read_only_layouts = {
+    constexpr std::array read_only_layouts = {
         VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL,
         VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_STENCIL_ATTACHMENT_OPTIMAL,
         VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL,
@@ -231,7 +234,7 @@ static inline bool IsImageLayoutDepthReadOnly(VkImageLayout layout) {
 }
 
 static inline bool IsImageLayoutStencilReadOnly(VkImageLayout layout) {
-    constexpr std::array<VkImageLayout, 7> read_only_layouts = {
+    constexpr std::array read_only_layouts = {
         VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL,
         VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_STENCIL_READ_ONLY_OPTIMAL,
         VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL,
@@ -293,6 +296,34 @@ static inline bool IsAdvanceBlendOperation(const VkBlendOp blend_op) {
     return (static_cast<int>(blend_op) >= VK_BLEND_OP_ZERO_EXT) && (static_cast<int>(blend_op) <= VK_BLEND_OP_BLUE_EXT);
 }
 
+// Helper for Dual-Source Blending
+static inline bool IsSecondaryColorInputBlendFactor(VkBlendFactor blend_factor) {
+    return (blend_factor == VK_BLEND_FACTOR_SRC1_COLOR || blend_factor == VK_BLEND_FACTOR_ONE_MINUS_SRC1_COLOR ||
+            blend_factor == VK_BLEND_FACTOR_SRC1_ALPHA || blend_factor == VK_BLEND_FACTOR_ONE_MINUS_SRC1_ALPHA);
+}
+
+// Check if size is in range
+static inline bool IsBetweenInclusive(VkDeviceSize value, VkDeviceSize min, VkDeviceSize max) {
+    return (value >= min) && (value <= max);
+}
+
+static inline bool IsBetweenInclusive(const VkExtent2D &value, const VkExtent2D &min, const VkExtent2D &max) {
+    return IsBetweenInclusive(value.width, min.width, max.width) && IsBetweenInclusive(value.height, min.height, max.height);
+}
+
+// Check if value is integer multiple of granularity
+static inline bool IsIntegerMultipleOf(VkDeviceSize value, VkDeviceSize granularity) {
+    if (granularity == 0) {
+        return value == 0;
+    } else {
+        return (value % granularity) == 0;
+    }
+}
+
+static inline bool IsIntegerMultipleOf(const VkOffset2D &value, const VkOffset2D &granularity) {
+    return IsIntegerMultipleOf(value.x, granularity.x) && IsIntegerMultipleOf(value.y, granularity.y);
+}
+
 // Perform a zero-tolerant modulo operation
 static inline VkDeviceSize SafeModulo(VkDeviceSize dividend, VkDeviceSize divisor) {
     VkDeviceSize result = 0;
@@ -310,6 +341,17 @@ static inline VkDeviceSize SafeDivision(VkDeviceSize dividend, VkDeviceSize divi
     return result;
 }
 
+// Only 32 bit fields should need a bit count
+static inline uint32_t GetBitSetCount(uint32_t field) {
+    std::bitset<32> view_bits(field);
+    return static_cast<uint32_t>(view_bits.count());
+}
+
+static inline uint32_t FullMipChainLevels(VkExtent3D extent) {
+    // uint cast applies floor()
+    return 1u + static_cast<uint32_t>(log2(std::max({extent.height, extent.width, extent.depth})));
+}
+
 extern "C" {
 #endif
 
@@ -348,60 +390,13 @@ static inline int u_ffs(int val) {
 #endif
 
 #ifdef __cplusplus
-// clang sets _MSC_VER to 1800 and _MSC_FULL_VER to 180000000, but we only want to clean up after MSVC.
-#if defined(_MSC_FULL_VER) && !defined(__clang__)
-// Minimum Visual Studio 2015 Update 2, or libc++ with C++17
-// But, before Visual Studio 2017 version 15.7, __cplusplus is not set
-// correctly. See:
-//   https://docs.microsoft.com/en-us/cpp/build/reference/zc-cplusplus?view=msvc-160
-// Also, according to commit e2a6c442cb1e4, SDKs older than NTDDI_WIN10_RS2 do not
-// support shared_mutex.
-#if _MSC_FULL_VER >= 190023918 && NTDDI_VERSION > NTDDI_WIN10_RS2 && (!defined(_LIBCPP_VERSION) || __cplusplus >= 201703)
-#define VVL_USE_SHARED_MUTEX 1
-#endif
-#elif __cplusplus >= 201703
-#define VVL_USE_SHARED_MUTEX 1
-#elif __cplusplus >= 201402
-#define VVL_USE_SHARED_TIMED_MUTEX 1
-#endif
-
-#if defined(VVL_USE_SHARED_MUTEX) || defined(VVL_USE_SHARED_TIMED_MUTEX)
 #include <shared_mutex>
-#endif
 
-class ReadWriteLock {
-  private:
-#if defined(VVL_USE_SHARED_MUTEX)
-    typedef std::shared_mutex Lock;
-#elif defined(VVL_USE_SHARED_TIMED_MUTEX)
-    typedef std::shared_timed_mutex Lock;
-#else
-    typedef std::mutex Lock;
-#endif
-
-  public:
-    void lock() { m_lock.lock(); }
-    bool try_lock() { return m_lock.try_lock(); }
-    void unlock() { m_lock.unlock(); }
-#if defined(VVL_USE_SHARED_MUTEX) || defined(VVL_USE_SHARED_TIMED_MUTEX)
-    void lock_shared() { m_lock.lock_shared(); }
-    bool try_lock_shared() { return m_lock.try_lock_shared(); }
-    void unlock_shared() { m_lock.unlock_shared(); }
-#else
-    void lock_shared() { lock(); }
-    bool try_lock_shared() { return try_lock(); }
-    void unlock_shared() { unlock(); }
-#endif
-  private:
-    Lock m_lock;
-};
-
-#if defined(VVL_USE_SHARED_MUTEX) || defined(VVL_USE_SHARED_TIMED_MUTEX)
-typedef std::shared_lock<ReadWriteLock> ReadLockGuard;
-#else
-typedef std::unique_lock<ReadWriteLock> ReadLockGuard;
-#endif
-typedef std::unique_lock<ReadWriteLock> WriteLockGuard;
+// Aliases to avoid excessive typing. We can't easily auto these away because
+// there are virtual methods in ValidationObject which return lock guards
+// and those cannot use return type deduction.
+typedef std::shared_lock<std::shared_mutex> ReadLockGuard;
+typedef std::unique_lock<std::shared_mutex> WriteLockGuard;
 
 // helper class for the very common case of getting and then locking a command buffer (or other state object)
 template <typename T, typename Guard>
@@ -502,7 +497,7 @@ class vl_concurrent_unordered_map {
         ReadLockGuard lock(locks[h].lock);
 
         auto itr = maps[h].find(key);
-        bool found = itr != maps[h].end();
+        const bool found = itr != maps[h].end();
 
         if (found) {
             return FindResult(true, itr->second);
@@ -516,7 +511,7 @@ class vl_concurrent_unordered_map {
         WriteLockGuard lock(locks[h].lock);
 
         auto itr = maps[h].find(key);
-        bool found = itr != maps[h].end();
+        const bool found = itr != maps[h].end();
 
         if (found) {
             auto ret = FindResult(true, itr->second);
@@ -570,9 +565,9 @@ class vl_concurrent_unordered_map {
 
     layer_data::unordered_map<Key, T, Hash> maps[BUCKETS];
     struct {
-        mutable ReadWriteLock lock;
+        mutable std::shared_mutex lock;
         // Put each lock on its own cache line to avoid false cache line sharing.
-        char padding[(-int(sizeof(ReadWriteLock))) & 63];
+        char padding[(-int(sizeof(std::shared_mutex))) & 63];
     } locks[BUCKETS];
 
     uint32_t ConcurrentMapHashObject(const Key &object) const {
