@@ -181,6 +181,7 @@ class CommandValidationOutputGenerator(OutputGenerator):
             write(self.commandRecordingList(), file=self.outFile)
             write(self.commandQueueTypeList(), file=self.outFile)
             write(self.commandRenderPassList(), file=self.outFile)
+            write(self.commandVideoCodingList(), file=self.outFile)
             write(self.commandBufferLevelList(), file=self.outFile)
             write(self.validateFunction(), file=self.outFile)
             write(self.dynamicFunction(), file=self.outFile)
@@ -251,7 +252,7 @@ static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedCommandNameList
     # List the enum for the dynamic command buffer status flags
     def dynamicTypeEnum(self):
         output = '''
-typedef enum CB_DYNAMIC_STATUS {\n'''
+typedef enum CBDynamicStatus {\n'''
         counter = 1
         for name in self.dynamic_states:
             state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
@@ -260,9 +261,9 @@ typedef enum CB_DYNAMIC_STATUS {\n'''
 
         output += '    CB_DYNAMIC_STATUS_NUM = ' + str(counter)
         output += '''
-} CB_DYNAMIC_STATUS;
+} CBDynamicStatus;
 
-typedef std::bitset<CB_DYNAMIC_STATUS_NUM> CBDynamicFlags;
+using CBDynamicFlags = std::bitset<CB_DYNAMIC_STATUS_NUM>;
 std::string DynamicStateString(CBDynamicFlags const &dynamic_state);
 struct VkPipelineDynamicStateCreateInfo;
 CBDynamicFlags MakeStaticStateMask(VkPipelineDynamicStateCreateInfo const *info);
@@ -371,6 +372,48 @@ static const std::array<CommandSupportedRenderPass, CMD_RANGE_SIZE> kGeneratedRe
         output += '}};'
         return output
 
+    #
+    # For each CMD_TYPE give a videocoding restriction and a *-videocoding VUID
+    def commandVideoCodingList(self):
+        output = '''
+enum CMD_VIDEO_CODING_TYPE {
+    CMD_VIDEO_CODING_INSIDE,
+    CMD_VIDEO_CODING_OUTSIDE,
+    CMD_VIDEO_CODING_BOTH
+};
+struct CommandSupportedVideoCoding {
+    CMD_VIDEO_CODING_TYPE videoCoding;
+    const char* vuid;
+};
+static const std::array<CommandSupportedVideoCoding, CMD_RANGE_SIZE> kGeneratedVideoCodingList = {{
+    {CMD_VIDEO_CODING_BOTH, kVUIDUndefined}, // CMD_NONE\n'''
+        for name, cmdinfo in sorted(self.commands.items()):
+            if name in self.alias_dict:
+                name = self.alias_dict[name]
+            video_coding_type = ''
+            video_coding = cmdinfo.elem.attrib.get('videocoding')
+            if video_coding is None:
+                video_coding = 'outside'
+            if video_coding == 'inside':
+                video_coding_type = 'CMD_VIDEO_CODING_INSIDE'
+            elif video_coding == 'outside':
+                video_coding_type = 'CMD_VIDEO_CODING_OUTSIDE'
+            elif video_coding != 'both':
+                print("videocoding attribute was %s and not known, need to update generation code", video_coding)
+                sys.exit(1)
+
+            # Only will be a VUID if not BOTH
+            if video_coding == 'both':
+                output += '    {CMD_VIDEO_CODING_BOTH, kVUIDUndefined},\n'
+            else:
+                vuid = 'VUID-' + name + '-videocoding'
+                if vuid not in self.valid_vuids:
+                    print("Warning: Could not find {} in validusage.json".format(vuid))
+                    vuid = vuid.replace('VUID-', 'UNASSIGNED-')
+                output += '    {' + video_coding_type + ', \"' + vuid + '\"},\n'
+        output += '}};'
+        return output
+
     #
     # For each CMD_TYPE give a buffer level restriction and add a *-bufferlevel VUID
     def commandBufferLevelList(self):
@@ -402,13 +445,13 @@ static const std::array<const char *, CMD_RANGE_SIZE> kGeneratedBufferLevelList
     def validateFunction(self):
         output = '''
 // Used to handle all the implicit VUs that are autogenerated from the registry
-bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cmd) const {
+bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE &cb_state, const CMD_TYPE cmd) const {
     bool skip = false;
     const char *caller_name = CommandTypeString(cmd);
 
     // Validate the given command being added to the specified cmd buffer,
     // flagging errors if CB is not in the recording state or if there's an issue with the Cmd ordering
-    switch (cb_state->state) {
+    switch (cb_state.state) {
         case CB_RECORDING:
             skip |= ValidateCmdSubpassState(cb_state, cmd);
             break;
@@ -421,7 +464,7 @@ bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cm
         default:
             assert(cmd != CMD_NONE);
             const auto error = kGeneratedMustBeRecordingList[cmd];
-            skip |= LogError(cb_state->commandBuffer(), error, "You must call vkBeginCommandBuffer() before this call to %s.",
+            skip |= LogError(cb_state.commandBuffer(), error, "You must call vkBeginCommandBuffer() before this call to %s.",
                             caller_name);
     }
 
@@ -437,6 +480,14 @@ bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cm
         skip |= InsideRenderPass(cb_state, caller_name, supportedRenderPass.vuid);
     }
 
+    // Validate if command is inside or outside a video coding scope if applicable
+    const auto supportedVideoCoding = kGeneratedVideoCodingList[cmd];
+    if (supportedVideoCoding.videoCoding == CMD_VIDEO_CODING_INSIDE) {
+        skip |= OutsideVideoCodingScope(cb_state, caller_name, supportedVideoCoding.vuid);
+    } else if (supportedVideoCoding.videoCoding == CMD_VIDEO_CODING_OUTSIDE) {
+        skip |= InsideVideoCodingScope(cb_state, caller_name, supportedVideoCoding.vuid);
+    }
+
     // Validate if command has to be recorded in a primary command buffer
     const auto supportedBufferLevel = kGeneratedBufferLevelList[cmd];
     if (supportedBufferLevel != nullptr) {
@@ -451,7 +502,7 @@ bool CoreChecks::ValidateCmd(const CMD_BUFFER_STATE *cb_state, const CMD_TYPE cm
     # List the enum for the dynamic command buffer status flags
     def dynamicFunction(self):
         output = '''
-static VkDynamicState ConvertToDynamicState(CB_DYNAMIC_STATUS flag) {
+static VkDynamicState ConvertToDynamicState(CBDynamicStatus flag) {
     switch (flag) {\n'''
         for name in self.dynamic_states:
             state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
@@ -463,7 +514,7 @@ static VkDynamicState ConvertToDynamicState(CB_DYNAMIC_STATUS flag) {
 }
 '''
         output += '''
-static CB_DYNAMIC_STATUS ConvertToCBDynamicStatus(VkDynamicState state) {
+static CBDynamicStatus ConvertToCBDynamicStatus(VkDynamicState state) {
     switch (state) {\n'''
         for name in self.dynamic_states:
             state_name = name[17:] # VK_DYNAMIC_STATE_LINE_WIDTH -> LINE_WIDTH
@@ -478,8 +529,9 @@ static CB_DYNAMIC_STATUS ConvertToCBDynamicStatus(VkDynamicState state) {
         output += '''
 std::string DynamicStateString(CBDynamicFlags const &dynamic_state) {
     std::string ret;
-    for (int index = 0; index < CB_DYNAMIC_STATUS_NUM; ++index) {
-        CB_DYNAMIC_STATUS status = static_cast<CB_DYNAMIC_STATUS>(index);
+    // enum is not zero based
+    for (int index = 1; index < CB_DYNAMIC_STATUS_NUM; ++index) {
+        CBDynamicStatus status = static_cast<CBDynamicStatus>(index);
         if (dynamic_state[status]) {
             if (!ret.empty()) ret.append("|");
             ret.append(string_VkDynamicState(ConvertToDynamicState(status)));
