@@ -143,7 +143,7 @@ def makeGenOpts(args):
             valid_usage_path  = args.scripts)
           ]
 
-    # Options for stateless validation source file
+    # Options for stateless validation header file
     genOpts['parameter_validation.h'] = [
           ParameterValidationOutputGenerator,
           ParameterValidationGeneratorOptions(
@@ -158,6 +158,21 @@ def makeGenOpts(args):
             valid_usage_path  = args.scripts)
           ]
 
+    # Options for stateless validation enum helper file
+    genOpts['enum_flag_bits.h'] = [
+          ParameterValidationOutputGenerator,
+          ParameterValidationGeneratorOptions(
+            conventions       = conventions,
+            filename          = 'enum_flag_bits.h',
+            directory         = directory,
+            versions          = featuresPat,
+            emitversions      = featuresPat,
+            addExtensions     = addExtensionsPat,
+            removeExtensions  = removeExtensionsPat,
+            emitExtensions    = emitExtensionsPat,
+            valid_usage_path  = args.scripts)
+          ]
+
     # Options for object_tracker code-generated validation routines
     genOpts['object_tracker.cpp'] = [
           ObjectTrackerOutputGenerator,
@@ -336,36 +351,6 @@ def makeGenOpts(args):
             helper_file_type  = 'typemap_helper_header')
         ]
 
-    # Helper file generator options for corechecks_optick_instrumentation.h
-    genOpts['corechecks_optick_instrumentation.h'] = [
-          HelperFileOutputGenerator,
-          HelperFileOutputGeneratorOptions(
-            conventions       = conventions,
-            filename          = 'corechecks_optick_instrumentation.h',
-            directory         = directory,
-            versions          = featuresPat,
-            emitversions      = featuresPat,
-            addExtensions     = addExtensionsPat,
-            removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'optick_instrumentation_header')
-        ]
-
-    # Helper file generator options for corechecks_optick_instrumentation.cpp
-    genOpts['corechecks_optick_instrumentation.cpp'] = [
-          HelperFileOutputGenerator,
-          HelperFileOutputGeneratorOptions(
-            conventions       = conventions,
-            filename          = 'corechecks_optick_instrumentation.cpp',
-            directory         = directory,
-            versions          = featuresPat,
-            emitversions      = featuresPat,
-            addExtensions     = addExtensionsPat,
-            removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'optick_instrumentation_source')
-        ]
-
     # Layer chassis related generation structs
     # Options for layer chassis header
     genOpts['chassis.h'] = [
