@@ -96,6 +96,8 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
         self.imageGatherOps = []
         self.imageSampleOps = []
         self.imageFetchOps = []
+        self.storageClassList = [] # list of storage classes
+        self.executionModel = []
         # Need range to be large as largest possible operand index
         self.imageOperandsParamCount = [[] for i in range(3)]
 
@@ -110,8 +112,7 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
 '''
 
     def commonParamSwitch(self, variableName):
-        return '''            break;
-        default:
+        return '''        default:
             break;
     }}
     return {};
@@ -164,6 +165,7 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
         if self.sourceFile:
             write('#include "vk_layer_data.h"', file=self.outFile)
             write('#include "spirv_grammar_helper.h"', file=self.outFile)
+            write('#include "shader_instruction.h"', file=self.outFile)
         elif self.headerFile:
             write('#pragma once', file=self.outFile)
             write('#include <cstdint>', file=self.outFile)
@@ -175,6 +177,7 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
         write(self.instructionTable(), file=self.outFile)
         write(self.atomicOperation(), file=self.outFile)
         write(self.groupOperation(), file=self.outFile)
+        write(self.storageClassHelper(), file=self.outFile)
         write(self.imageOperation(), file=self.outFile)
         write(self.parameterHelper(), file=self.outFile)
         write(self.stringHelper(), file=self.outFile)
@@ -216,9 +219,22 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
                         if enum['value'] not in values:
                             self.imageOperandsParamCount[count].append(enum['enumerant'])
                             values.append(enum['value'])
+                if operandKind['kind'] == 'StorageClass':
+                    values = [] # prevent alias from being duplicatd
+                    for enum in operandKind['enumerants']:
+                        if enum['value'] not in values:
+                            self.storageClassList.append(enum['enumerant'])
+                            values.append(enum['value'])
+                if operandKind['kind'] == 'ExecutionModel':
+                    values = [] # prevent alias from being duplicatd
+                    for enum in operandKind['enumerants']:
+                        if enum['value'] not in values:
+                            self.executionModel.append(enum['enumerant'])
+                            values.append(enum['value'])
 
             for instruction in instructions:
                 opname = instruction['opname']
+                opcode = instruction['opcode']
                 if 'capabilities' in instruction:
                     notSupported = True
                     for capability in instruction['capabilities']:
@@ -230,13 +246,14 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
 
                 # Nice side effect of using a dict here is alias opcodes will be last in the grammar file
                 # ex: OpTypeAccelerationStructureNV will be replaced by OpTypeAccelerationStructureKHR
-                self.opcodes[instruction['opcode']] = {
+                self.opcodes[opcode] = {
                     'name' : opname,
                     'hasType' : "false",
                     'hasResult' : "false",
                     'memoryScopePosition' : 0,
                     'executionScopePosition' : 0,
                     'imageOperandsPosition' : 0,
+                    'storageClassPosition' : 0,
                 }
 
                 if instruction['class'] == 'Atomic':
@@ -252,21 +269,23 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
                 if 'operands' in instruction:
                     for index, operand in enumerate(instruction['operands']):
                         if operand['kind'] == 'IdResultType':
-                            self.opcodes[instruction['opcode']]['hasType'] = "true"
+                            self.opcodes[opcode]['hasType'] = "true"
                         if operand['kind'] == 'IdResult':
-                            self.opcodes[instruction['opcode']]['hasResult'] = "true"
+                            self.opcodes[opcode]['hasResult'] = "true"
                         # some instructions have both types of IdScope
                         # OpReadClockKHR has the wrong 'name' as 'Scope'
                         if operand['kind'] == 'IdScope':
                             if operand['name'] == '\'Execution\'' or operand['name'] == '\'Scope\'':
-                                self.opcodes[instruction['opcode']]['executionScopePosition'] = index + 1
+                                self.opcodes[opcode]['executionScopePosition'] = index + 1
                             elif operand['name'] == '\'Memory\'':
-                                self.opcodes[instruction['opcode']]['memoryScopePosition'] = index + 1
+                                self.opcodes[opcode]['memoryScopePosition'] = index + 1
                             else:
                                 print("Error: unknown operand {} not handled correctly\n".format(opname))
                                 sys.exit(1)
                         if operand['kind'] == 'ImageOperands':
-                            self.opcodes[instruction['opcode']]['imageOperandsPosition'] = index + 1
+                            self.opcodes[opcode]['imageOperandsPosition'] = index + 1
+                        if operand['kind'] == 'StorageClass':
+                            self.opcodes[opcode]['storageClassPosition'] = index + 1
 
     #
     # Generate table for each opcode instruction
@@ -326,6 +345,23 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
 
         return output;
     #
+    # Get storage class from instruction
+    def storageClassHelper(self):
+        output = ''
+        if self.sourceFile:
+            output += 'spv::StorageClass Instruction::StorageClass() const {\n'
+            output += '    spv::StorageClass storage_class = spv::StorageClassMax;\n'
+            output += '    switch (Opcode()) {\n'
+            for opcode, info in sorted(self.opcodes.items()):
+                operand = info['storageClassPosition']
+                if operand == 0:
+                    continue
+                output += '        case spv::{}:\n'.format(info['name'])
+                output += '            storage_class = static_cast<spv::StorageClass>(Word({}));\n'.format(info['storageClassPosition'])
+                output += '            break;\n'
+            output += self.commonParamSwitch('storage_class')
+        return output;
+    #
     # Generate functions for image operations
     def imageOperation(self):
         output = ''
@@ -441,6 +477,8 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
         output = ''
         if self.headerFile:
             output =  'const char* string_SpvOpcode(uint32_t opcode);\n'
+            output +=  'const char* string_SpvStorageClass(uint32_t storage_class);\n'
+            output +=  'const char* string_SpvExecutionModel(uint32_t execution_model);\n'
         elif self.sourceFile:
             output =  'const char* string_SpvOpcode(uint32_t opcode) {\n'
             output += '    auto format_info = kInstructionTable.find(opcode);\n'
@@ -449,5 +487,23 @@ class SpirvGrammarHelperOutputGenerator(OutputGenerator):
             output += '    } else {\n'
             output += '        return \"Unhandled Opcode\";\n'
             output += '    }\n'
-            output += '};'
+            output += '};\n'
+            output += '\nconst char* string_SpvStorageClass(uint32_t storage_class) {\n'
+            output += '    switch(storage_class) {\n'
+            for storageClass in self.storageClassList:
+                output += '        case spv::StorageClass{}:\n'.format(storageClass)
+                output += '            return \"{}\";\n'.format(storageClass)
+            output += '        default:\n'
+            output += '            return \"unknown\";\n'
+            output += '    }\n'
+            output += '};\n'
+            output += '\nconst char* string_SpvExecutionModel(uint32_t execution_model) {\n'
+            output += '    switch(execution_model) {\n'
+            for executionModel in self.executionModel:
+                output += '        case spv::ExecutionModel{}:\n'.format(executionModel)
+                output += '            return \"{}\";\n'.format(executionModel)
+            output += '        default:\n'
+            output += '            return \"unknown\";\n'
+            output += '    }\n'
+            output += '};\n'
         return output
