@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2022 The Khronos Group Inc.
- * Copyright (c) 2015-2022 Valve Corporation
- * Copyright (c) 2015-2022 LunarG, Inc.
- * Copyright (c) 2015-2022 Google, Inc.
+ * Copyright (c) 2015-2023 The Khronos Group Inc.
+ * Copyright (c) 2015-2023 Valve Corporation
+ * Copyright (c) 2015-2023 LunarG, Inc.
+ * Copyright (c) 2015-2023 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -27,11 +27,7 @@
 #ifndef VKLAYERTEST_H
 #define VKLAYERTEST_H
 
-#ifdef ANDROID
-#include "vulkan_wrapper.h"
-#else
 #include <vulkan/vulkan.h>
-#endif
 
 #include "layers/vk_lunarg_device_profile_api_layer.h"
 #include "vk_layer_settings_ext.h"
@@ -67,8 +63,6 @@ using std::vector;
 // Mesh and VertexFormat Data
 //--------------------------------------------------------------------------------------
 
-static const char kSkipPrefix[] = "             TEST SKIPPED:";
-
 enum BsoFailSelect {
     BsoFailNone,
     BsoFailLineWidth,
@@ -184,12 +178,28 @@ static char const bindStateFragSubpassLoadInputText[] = R"glsl(
         }
     )glsl";
 
+[[maybe_unused]] static const char *bindStateRTShaderText = R"glsl(
+        #version 460
+        #extension GL_EXT_ray_tracing : require
+        void main() {}
+    )glsl";
+
 // Static arrays helper
 template <class ElementT, size_t array_size>
 size_t size(ElementT (&)[array_size]) {
     return array_size;
 }
 
+template <class ElementT, size_t array_size>
+uint32_t size32(ElementT (&)[array_size]) {
+    return static_cast<uint32_t>(array_size);
+}
+
+template <class Container>
+uint32_t size32(const Container &c) {
+    return static_cast<uint32_t>(c.size());
+}
+
 // Format search helper
 VkFormat FindSupportedDepthOnlyFormat(VkPhysicalDevice phy);
 VkFormat FindSupportedStencilOnlyFormat(VkPhysicalDevice phy);
@@ -218,9 +228,6 @@ VkImageViewCreateInfo SafeSaneImageViewCreateInfo(VkImage image, VkFormat format
 
 VkImageViewCreateInfo SafeSaneImageViewCreateInfo(const VkImageObj &image, VkFormat format, VkImageAspectFlags aspect_mask);
 
-// Helper for checking createRenderPass2 support and adding related extensions.
-bool CheckCreateRenderPass2Support(VkRenderFramework *renderFramework, std::vector<const char *> &device_extension_names);
-
 // Helper for checking timeline semaphore support and initializing
 bool CheckTimelineSemaphoreSupportAndInitState(VkRenderFramework *renderFramework);
 
@@ -276,6 +283,28 @@ class VkLayerTest : public VkRenderFramework {
         return GetPhysicalDeviceProperties2(props2);
     }
 
+    template <typename Proc, bool assert_proc = true>
+    [[nodiscard]] const Proc GetInstanceProcAddr(const char *proc_name) noexcept {
+        static_assert(std::is_pointer_v<Proc>);
+
+        auto proc = reinterpret_cast<Proc>(vk::GetInstanceProcAddr(instance(), proc_name));
+        if constexpr (assert_proc) {
+            assert(proc);
+        }
+        return proc;
+    }
+
+    template <typename Proc, bool assert_proc = true>
+    [[nodiscard]] const Proc GetDeviceProcAddr(const char *proc_name) noexcept {
+        static_assert(std::is_pointer_v<Proc>);
+
+        auto proc = reinterpret_cast<Proc>(vk::GetDeviceProcAddr(device(), proc_name));
+        if constexpr (assert_proc) {
+            assert(proc);
+        }
+        return proc;
+    }
+
     bool IsDriver(VkDriverId driver_id);
 
   protected:
@@ -494,7 +523,7 @@ struct CreatePipelineHelper {
     std::unique_ptr<VkShaderObj> vs_;
     std::unique_ptr<VkShaderObj> fs_;
     VkLayerTest &layer_test_;
-    layer_data::optional<VkGraphicsPipelineLibraryCreateInfoEXT> gpl_info;
+    std::optional<VkGraphicsPipelineLibraryCreateInfoEXT> gpl_info;
     CreatePipelineHelper(VkLayerTest &test, uint32_t color_attachments_count = 1u);
     ~CreatePipelineHelper();
 
@@ -787,7 +816,7 @@ class BarrierQueueFamilyTestHelper : public BarrierQueueFamilyBase {
     // Init with queue families non-null for CONCURRENT sharing mode (which requires them)
     void Init(std::vector<uint32_t> *families, bool image_memory = true, bool buffer_memory = true);
 
-    void operator()(std::string img_err, std::string buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
+    void operator()(const std::string &img_err, const std::string &buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
                     uint32_t dst = VK_QUEUE_FAMILY_IGNORED, uint32_t queue_family_index = kInvalidQueueFamily,
                     Modifier mod = Modifier::NONE);
 
@@ -806,7 +835,7 @@ class Barrier2QueueFamilyTestHelper : public BarrierQueueFamilyBase {
     // Init with queue families non-null for CONCURRENT sharing mode (which requires them)
     void Init(std::vector<uint32_t> *families, bool image_memory = true, bool buffer_memory = true);
 
-    void operator()(std::string img_err, std::string buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
+    void operator()(const std::string &img_err, const std::string &buf_err = "", uint32_t src = VK_QUEUE_FAMILY_IGNORED,
                     uint32_t dst = VK_QUEUE_FAMILY_IGNORED, uint32_t queue_family_index = kInvalidQueueFamily,
                     Modifier mod = Modifier::NONE);
 
@@ -891,15 +920,15 @@ void AllocateDisjointMemory(VkDeviceObj *device, PFN_vkGetImageMemoryRequirement
 
 void NegHeightViewportTests(VkDeviceObj *m_device, VkCommandBufferObj *m_commandBuffer, ErrorMonitor *m_errorMonitor);
 
-void CreateSamplerTest(VkLayerTest &test, const VkSamplerCreateInfo *pCreateInfo, std::string code = "");
+void CreateSamplerTest(VkLayerTest &test, const VkSamplerCreateInfo *pCreateInfo, const std::string &code = "");
 
-void CreateBufferTest(VkLayerTest &test, const VkBufferCreateInfo *pCreateInfo, std::string code = "");
+void CreateBufferTest(VkLayerTest &test, const VkBufferCreateInfo *pCreateInfo, const std::string &code = "");
 
-void CreateImageTest(VkLayerTest &test, const VkImageCreateInfo *pCreateInfo, std::string code = "");
+void CreateImageTest(VkLayerTest &test, const VkImageCreateInfo *pCreateInfo, const std::string &code = "");
 
 void CreateBufferViewTest(VkLayerTest &test, const VkBufferViewCreateInfo *pCreateInfo, const std::vector<std::string> &codes);
 
-void CreateImageViewTest(VkLayerTest &test, const VkImageViewCreateInfo *pCreateInfo, std::string code = "");
+void CreateImageViewTest(VkLayerTest &test, const VkImageViewCreateInfo *pCreateInfo, const std::string &code = "");
 
 bool InitFrameworkForRayTracingTest(VkRenderFramework *framework, bool is_khr, VkPhysicalDeviceFeatures2KHR *features2 = nullptr,
                                     VkValidationFeaturesEXT *enabled_features = nullptr, bool mockicd_valid = false);
@@ -907,5 +936,8 @@ bool InitFrameworkForRayTracingTest(VkRenderFramework *framework, bool is_khr, V
 void GetSimpleGeometryForAccelerationStructureTests(const VkDeviceObj &device, VkBufferObj *vbo, VkBufferObj *ibo,
                                                     VkGeometryNV *geometry, VkDeviceSize offset = 0, bool buffer_device_address = false);
 
+std::pair<VkBufferObj &&, VkAccelerationStructureGeometryKHR> GetSimpleAABB(const VkDeviceObj &device,
+                                                                            uint32_t vk_api_version = VK_API_VERSION_1_2);
+
 void print_android(const char *c);
 #endif  // VKLAYERTEST_H
