@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2019 The Khronos Group Inc.
- * Copyright (c) 2015-2019 Valve Corporation
- * Copyright (c) 2015-2019 LunarG, Inc.
+ * Copyright (c) 2015-2022 The Khronos Group Inc.
+ * Copyright (c) 2015-2022 Valve Corporation
+ * Copyright (c) 2015-2023 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,44 +22,12 @@
 
 #include "vktestframework.h"
 #include "vkrenderframework.h"
-
-// For versions prior to VS 2015, suppress the warning
-// caused by the inconsistent redefinition of snprintf
-// between a vulkan header and a glslang header.
-#if (defined(_MSC_VER) && _MSC_VER < 1900 /*vs2015*/)
-#pragma warning(push)
-#pragma warning(disable : 4005)
-#endif
-// TODO FIXME remove this once glslang doesn't define this
-#undef BadValue
-#include "SPIRV/GlslangToSpv.h"
-#include "SPIRV/SPVRemapper.h"
-#if (defined(_MSC_VER) && _MSC_VER < 1900 /*vs2015*/)
-#pragma warning(pop)
-#endif
-#include <limits.h>
+#include "glslang/SPIRV/GlslangToSpv.h"
+#include "glslang/SPIRV/SPVRemapper.h"
+#include <filesystem>
+#include <climits>
 #include <cmath>
 
-#if defined(PATH_MAX) && !defined(MAX_PATH)
-#define MAX_PATH PATH_MAX
-#endif
-
-#ifdef _WIN32
-#define ERR_EXIT(err_msg, err_class)                 \
-    do {                                             \
-        MessageBox(NULL, err_msg, err_class, MB_OK); \
-        exit(1);                                     \
-    } while (0)
-#else  // _WIN32
-
-#define ERR_EXIT(err_msg, err_class) \
-    do {                             \
-        printf(err_msg);             \
-        fflush(stdout);              \
-        exit(1);                     \
-    } while (0)
-#endif  // _WIN32
-
 // Command-line options
 enum TOptions {
     EOptionNone = 0x000,
@@ -108,9 +76,117 @@ int fopen_s(FILE **pFile, const char *filename, const char *mode) {
 
 #endif
 
+// Certain VK_* environment variables accept lists.
+// Return a vector of std::string containing each member in the list.
+//
+// EX input:
+//  export VK_DRIVER_FILES=/intel.json:/amd.json
+//  set VK_DRIVER_FILES=\nvidia.json;\mesa.json
+static std::vector<std::string> GetVkEnvironmentVariable(const char *env_var) {
+    const std::string str = GetEnvironment(env_var);
+    if (str.empty()) {
+        return {};
+    }
+
+    // Loader uses standard OS path separators per platform
+    constexpr char delimiter =
+#ifdef _WIN32
+        ';';
+#else
+        ':';
+#endif
+
+    std::vector<std::string> items;
+    std::string::size_type start = 0;
+
+    std::string::size_type pos = str.find_first_of(delimiter, start);
+    std::string::size_type length = pos;
+    while (pos != std::string::npos) {
+        // emplace uses std::substr which takes length from start
+        items.emplace_back(str, start, length);
+
+        start = pos + 1;
+
+        pos = str.find_first_of(delimiter, start);
+
+        length = pos - start;
+    }
+    items.emplace_back(str, start);
+
+    return items;
+}
+
+static void CheckEnvironmentVariables() {
+    for (const char *env_var : {"VK_DRIVER_FILES", "VK_ICD_FILENAMES"}) {
+        const std::vector<std::string> driver_files = GetVkEnvironmentVariable(env_var);
+        for (const std::string &driver_file : driver_files) {
+            const std::filesystem::path icd_file(driver_file);
+
+            // TODO: Error check relative paths (platform dependent)
+            if (icd_file.is_relative()) {
+                continue;
+            }
+
+            std::string user_provided;
+            user_provided += "\n\n";
+            user_provided += env_var;
+            user_provided += " = ";
+            user_provided += driver_file;
+
+            if (!std::filesystem::exists(icd_file)) {
+                std::cerr << "Invalid " << env_var << "! File doesn't exist!" << user_provided << std::endl;
+                std::exit(EXIT_FAILURE);
+            }
+
+            if (icd_file.extension() != ".json") {
+                std::cerr << "Invalid " << env_var << "! " << env_var << " must be a json file!\n" << user_provided << std::endl;
+                std::exit(EXIT_FAILURE);
+            }
+        }
+    }
+
+    const std::vector<std::string> vk_layer_paths = GetVkEnvironmentVariable("VK_LAYER_PATH");
+    bool found_json = false;
+    for (const std::string &layer_path : vk_layer_paths) {
+        const std::filesystem::path layer_dir(layer_path);
+
+        // TODO: Error check relative paths (platform dependent)
+        if (layer_dir.is_relative()) {
+            continue;
+        }
+        const std::string user_provided = "\n\nVK_LAYER_PATH = " + layer_path;
+
+        if (!std::filesystem::exists(layer_dir)) {
+            std::cerr << "Invalid VK_LAYER_PATH! Directory " << layer_dir << " doesn't exist!" << user_provided << std::endl;
+            std::exit(EXIT_FAILURE);
+        }
+
+        if (!std::filesystem::is_directory(layer_dir)) {
+            std::cerr << "Invalid VK_LAYER_PATH! " << layer_dir << " must be a directory!" << user_provided << std::endl;
+            std::exit(EXIT_FAILURE);
+        }
+
+        for (auto const &dir_entry : std::filesystem::directory_iterator{layer_dir}) {
+            if (dir_entry.path().filename() == "VkLayer_khronos_validation.json") {
+                found_json = true;
+                break;
+            }
+        }
+    }
+
+    if (!found_json) {
+        std::cerr << "Invalid VK_LAYER_PATH! VK_LAYER_PATH directory must contain VkLayer_khronos_validation.json!"
+                  << GetEnvironment("VK_LAYER_PATH") << std::endl;
+        std::exit(EXIT_FAILURE);
+    }
+}
+
 // Set up environment for GLSL compiler
 // Must be done once per process
 void TestEnvironment::SetUp() {
+    // Helps ensure common developer environment variables are set correctly
+    CheckEnvironmentVariables();
+
     // Initialize GLSL to SPV compiler utility
     glslang::InitializeProcess();
 
@@ -129,9 +205,9 @@ VkTestFramework::~VkTestFramework() {}
 bool VkTestFramework::m_canonicalize_spv = false;
 bool VkTestFramework::m_strip_spv = false;
 bool VkTestFramework::m_do_everything_spv = false;
-bool VkTestFramework::m_devsim_layer = false;
 int VkTestFramework::m_width = 0;
 int VkTestFramework::m_height = 0;
+int VkTestFramework::m_phys_device_index = -1;
 
 bool VkTestFramework::optionMatch(const char *option, char *optionLine) {
     if (strncmp(option, optionLine, strlen(option)) == 0)
@@ -148,9 +224,9 @@ void VkTestFramework::InitArgs(int *argc, char *argv[]) {
             m_strip_spv = true;
         else if (optionMatch("--canonicalize-SPV", argv[i]))
             m_canonicalize_spv = true;
-        else if (optionMatch("--devsim", argv[i]))
-            m_devsim_layer = true;
-        else if (optionMatch("--help", argv[i]) || optionMatch("-h", argv[i])) {
+        else if (optionMatch("--device-index", argv[i]) && ((i + 1) < *argc)) {
+            m_phys_device_index = std::atoi(argv[++i]);
+        } else if (optionMatch("--help", argv[i]) || optionMatch("-h", argv[i])) {
             printf("\nOther options:\n");
             printf(
                 "\t--show-images\n"
@@ -176,6 +252,11 @@ void VkTestFramework::InitArgs(int *argc, char *argv[]) {
             printf(
                 "\t--canonicalize-SPV\n"
                 "\t\tRemap SPIR-V ids before submission to aid compression.\n");
+            printf(
+                "\t--device-index <physical device index>\n"
+                "\t\tIndex into VkPhysicalDevice array returned from vkEnumeratePhysicalDevices.\n"
+                "\t\tThe default behavior is to automatically choose \"the most reasonable device.\"\n"
+                "\t\tAn invalid index (i.e., outside the range [0, *pPhysicalDeviceCount)) will result in the default behavior\n");
             exit(0);
         } else {
             printf("\nUnrecognized option: %s\n", argv[i]);
@@ -639,7 +720,7 @@ char **VkTestFramework::ReadFileData(const char *fileName) {
         ptr_len += (len);
         if (count < len) {
             if (count == 0) {
-                m_num_shader_strings = (i + 1);
+                m_num_shader_strings = (static_cast<int>(i) + 1);
                 break;
             }
             len = count;
@@ -710,23 +791,23 @@ EShLanguage VkTestFramework::FindLanguage(const VkShaderStageFlagBits shader_typ
         case VK_SHADER_STAGE_COMPUTE_BIT:
             return EShLangCompute;
 
-        case VK_SHADER_STAGE_RAYGEN_BIT_NV:
-            return EShLangRayGenNV;
+        case VK_SHADER_STAGE_RAYGEN_BIT_KHR:
+            return EShLangRayGen;
 
-        case VK_SHADER_STAGE_ANY_HIT_BIT_NV:
-            return EShLangAnyHitNV;
+        case VK_SHADER_STAGE_ANY_HIT_BIT_KHR:
+            return EShLangAnyHit;
 
-        case VK_SHADER_STAGE_CLOSEST_HIT_BIT_NV:
-            return EShLangClosestHitNV;
+        case VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR:
+            return EShLangClosestHit;
 
-        case VK_SHADER_STAGE_MISS_BIT_NV:
-            return EShLangMissNV;
+        case VK_SHADER_STAGE_MISS_BIT_KHR:
+            return EShLangMiss;
 
-        case VK_SHADER_STAGE_INTERSECTION_BIT_NV:
-            return EShLangIntersectNV;
+        case VK_SHADER_STAGE_INTERSECTION_BIT_KHR:
+            return EShLangIntersect;
 
-        case VK_SHADER_STAGE_CALLABLE_BIT_NV:
-            return EShLangCallableNV;
+        case VK_SHADER_STAGE_CALLABLE_BIT_KHR:
+            return EShLangCallable;
 
         case VK_SHADER_STAGE_TASK_BIT_NV:
             return EShLangTaskNV;
@@ -744,7 +825,7 @@ EShLanguage VkTestFramework::FindLanguage(const VkShaderStageFlagBits shader_typ
 // Return value of false means an error was encountered.
 //
 bool VkTestFramework::GLSLtoSPV(VkPhysicalDeviceLimits const *const device_limits, const VkShaderStageFlagBits shader_type,
-                                const char *pshader, std::vector<unsigned int> &spirv, bool debug, uint32_t spirv_minor_version) {
+                                const char *pshader, std::vector<uint32_t> &spirv, bool debug, const spv_target_env spv_env) {
     glslang::TProgram program;
     const char *shaderStrings[1];
 
@@ -763,27 +844,9 @@ bool VkTestFramework::GLSLtoSPV(VkPhysicalDeviceLimits const *const device_limit
 
     EShLanguage stage = FindLanguage(shader_type);
     glslang::TShader *shader = new glslang::TShader(stage);
-    switch (spirv_minor_version) {
-        default:
-        case 0:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_0);
-            break;
-        case 1:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_1);
-            break;
-        case 2:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_2);
-            break;
-        case 3:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_3);
-            break;
-        case 4:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_4);
-            break;
-        case 5:
-            shader->setEnvTarget(glslang::EShTargetSpv, glslang::EShTargetSpv_1_5);
-            break;
-    }
+    VkShaderObj::GlslangTargetEnv glslang_env(spv_env);
+    shader->setEnvTarget(glslang::EshTargetSpv, glslang_env);
+    shader->setEnvClient(glslang::EShClientVulkan, glslang_env);
 
     shaderStrings[0] = pshader;
     shader->setStrings(shaderStrings, 1);
@@ -848,7 +911,7 @@ bool VkTestFramework::GLSLtoSPV(VkPhysicalDeviceLimits const *const device_limit
 // Return value of false means an error was encountered.
 //
 bool VkTestFramework::ASMtoSPV(const spv_target_env target_env, const uint32_t options, const char *pasm,
-                               std::vector<unsigned int> &spv) {
+                               std::vector<uint32_t> &spv) {
     spv_binary binary;
     spv_diagnostic diagnostic = nullptr;
     spv_context context = spvContextCreate(target_env);
